import type { DidCommService, ConnectionRecord } from '../modules/connections';
import type { OutboundTransporter } from '../transport/OutboundTransporter';
import type { OutboundMessage, OutboundPackage, WireMessage } from '../types';
import type { AgentMessage } from './AgentMessage';
import type { EnvelopeKeys } from './EnvelopeService';
import { DID_COMM_TRANSPORT_QUEUE } from '../constants';
import { Logger } from '../logger';
import { MessageRepository } from '../storage/MessageRepository';
import { EnvelopeService } from './EnvelopeService';
import { TransportService } from './TransportService';
export declare class MessageSender {
    private envelopeService;
    private transportService;
    private messageRepository;
    private logger;
    private _outboundTransporter?;
    constructor(envelopeService: EnvelopeService, transportService: TransportService, messageRepository: MessageRepository, logger: Logger);
    setOutboundTransporter(outboundTransporter: OutboundTransporter): void;
    get outboundTransporter(): OutboundTransporter | undefined;
    packMessage({ keys, message, endpoint, }: {
        keys: EnvelopeKeys;
        message: AgentMessage;
        endpoint: string;
    }): Promise<OutboundPackage>;
    private sendMessageToSession;
    sendPackage({ connection, packedMessage, }: {
        connection: ConnectionRecord;
        packedMessage: WireMessage;
    }): Promise<void>;
    sendMessage(outboundMessage: OutboundMessage): Promise<void>;
    sendMessageToService({ message, service, senderKey, returnRoute, }: {
        message: AgentMessage;
        service: DidCommService;
        senderKey: string;
        returnRoute?: boolean;
    }): Promise<void>;
}
export declare function isDidCommTransportQueue(serviceEndpoint: string): serviceEndpoint is typeof DID_COMM_TRANSPORT_QUEUE;
