"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isDidCommTransportQueue = exports.MessageSender = void 0;
const tsyringe_1 = require("tsyringe");
const constants_1 = require("../constants");
const TransportDecorator_1 = require("../decorators/transport/TransportDecorator");
const error_1 = require("../error");
const EnvelopeService_1 = require("./EnvelopeService");
const TransportService_1 = require("./TransportService");
let MessageSender = class MessageSender {
    constructor(envelopeService, transportService, messageRepository, logger) {
        this.envelopeService = envelopeService;
        this.transportService = transportService;
        this.messageRepository = messageRepository;
        this.logger = logger;
    }
    setOutboundTransporter(outboundTransporter) {
        this._outboundTransporter = outboundTransporter;
    }
    get outboundTransporter() {
        return this._outboundTransporter;
    }
    async packMessage({ keys, message, endpoint, }) {
        const wireMessage = await this.envelopeService.packMessage(message, keys);
        return {
            payload: wireMessage,
            responseRequested: message.hasAnyReturnRoute(),
            endpoint,
        };
    }
    async sendMessageToSession(session, message) {
        this.logger.debug(`Existing ${session.type} transport session has been found.`, {
            keys: session.keys,
        });
        if (!session.keys) {
            throw new error_1.AriesFrameworkError(`There are no keys for the given ${session.type} transport session.`);
        }
        const wireMessage = await this.envelopeService.packMessage(message, session.keys);
        await session.send(wireMessage);
    }
    async sendPackage({ connection, packedMessage, }) {
        var e_1, _a;
        var _b;
        // Try to send to already open session
        const session = this.transportService.findSessionByConnectionId(connection.id);
        if ((_b = session === null || session === void 0 ? void 0 : session.inboundMessage) === null || _b === void 0 ? void 0 : _b.hasReturnRouting()) {
            try {
                await session.send(packedMessage);
                return;
            }
            catch (error) {
                this.logger.info(`Sending packed message via session failed with error: ${error.message}.`, error);
            }
        }
        // Retrieve DIDComm services
        const allServices = this.transportService.findDidCommServices(connection);
        const reachableServices = allServices.filter((s) => !isDidCommTransportQueue(s.serviceEndpoint));
        const queueService = allServices.find((s) => isDidCommTransportQueue(s.serviceEndpoint));
        this.logger.debug(`Found ${allServices.length} services for message to connection '${connection.id}' (${connection.theirLabel})`);
        if (!this.outboundTransporter) {
            throw new error_1.AriesFrameworkError('Agent has no outbound transporter!');
        }
        try {
            // Loop trough all available services and try to send the message
            for (var reachableServices_1 = __asyncValues(reachableServices), reachableServices_1_1; reachableServices_1_1 = await reachableServices_1.next(), !reachableServices_1_1.done;) {
                const service = reachableServices_1_1.value;
                this.logger.debug(`Sending outbound message to service:`, { service });
                try {
                    await this.outboundTransporter.sendMessage({
                        payload: packedMessage,
                        endpoint: service.serviceEndpoint,
                    });
                    return;
                }
                catch (error) {
                    this.logger.debug(`Sending outbound message to service with id ${service.id} failed with the following error:`, {
                        message: error.message,
                        error: error,
                    });
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (reachableServices_1_1 && !reachableServices_1_1.done && (_a = reachableServices_1.return)) await _a.call(reachableServices_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        // We didn't succeed to send the message over open session, or directly to serviceEndpoint
        // If the other party shared a queue service endpoint in their did doc we queue the message
        if (queueService) {
            this.logger.debug(`Queue packed message for connection ${connection.id} (${connection.theirLabel})`);
            this.messageRepository.add(connection.id, packedMessage);
            return;
        }
        // Message is undeliverable
        this.logger.error(`Message is undeliverable to connection ${connection.id} (${connection.theirLabel})`, {
            message: packedMessage,
            connection,
        });
        throw new error_1.AriesFrameworkError(`Message is undeliverable to connection ${connection.id} (${connection.theirLabel})`);
    }
    async sendMessage(outboundMessage) {
        var e_2, _a;
        var _b;
        if (!this.outboundTransporter) {
            throw new error_1.AriesFrameworkError('Agent has no outbound transporter!');
        }
        const { connection, payload } = outboundMessage;
        this.logger.debug('Send outbound message', {
            message: payload,
            connectionId: connection.id,
        });
        // Try to send to already open session
        const session = this.transportService.findSessionByConnectionId(connection.id);
        if ((_b = session === null || session === void 0 ? void 0 : session.inboundMessage) === null || _b === void 0 ? void 0 : _b.hasReturnRouting(payload.threadId)) {
            try {
                await this.sendMessageToSession(session, payload);
                return;
            }
            catch (error) {
                this.logger.info(`Sending an outbound message via session failed with error: ${error.message}.`, error);
            }
        }
        // Retrieve DIDComm services
        const allServices = this.transportService.findDidCommServices(connection);
        const reachableServices = allServices.filter((s) => !isDidCommTransportQueue(s.serviceEndpoint));
        const queueService = allServices.find((s) => isDidCommTransportQueue(s.serviceEndpoint));
        this.logger.debug(`Found ${allServices.length} services for message to connection '${connection.id}' (${connection.theirLabel})`);
        try {
            // Loop trough all available services and try to send the message
            for (var reachableServices_2 = __asyncValues(reachableServices), reachableServices_2_1; reachableServices_2_1 = await reachableServices_2.next(), !reachableServices_2_1.done;) {
                const service = reachableServices_2_1.value;
                try {
                    // Enable return routing if the
                    const shouldUseReturnRoute = !this.transportService.hasInboundEndpoint(connection.didDoc);
                    await this.sendMessageToService({
                        message: payload,
                        service,
                        senderKey: connection.verkey,
                        returnRoute: shouldUseReturnRoute,
                    });
                    return;
                }
                catch (error) {
                    this.logger.debug(`Sending outbound message to service with id ${service.id} failed with the following error:`, {
                        message: error.message,
                        error: error,
                    });
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (reachableServices_2_1 && !reachableServices_2_1.done && (_a = reachableServices_2.return)) await _a.call(reachableServices_2);
            }
            finally { if (e_2) throw e_2.error; }
        }
        // We didn't succeed to send the message over open session, or directly to serviceEndpoint
        // If the other party shared a queue service endpoint in their did doc we queue the message
        if (queueService) {
            this.logger.debug(`Queue message for connection ${connection.id} (${connection.theirLabel})`);
            const keys = {
                recipientKeys: queueService.recipientKeys,
                routingKeys: queueService.routingKeys || [],
                senderKey: connection.verkey,
            };
            const wireMessage = await this.envelopeService.packMessage(payload, keys);
            this.messageRepository.add(connection.id, wireMessage);
            return;
        }
        // Message is undeliverable
        this.logger.error(`Message is undeliverable to connection ${connection.id} (${connection.theirLabel})`, {
            message: payload,
            connection,
        });
        throw new error_1.AriesFrameworkError(`Message is undeliverable to connection ${connection.id} (${connection.theirLabel})`);
    }
    async sendMessageToService({ message, service, senderKey, returnRoute, }) {
        if (!this.outboundTransporter) {
            throw new error_1.AriesFrameworkError('Agent has no outbound transporter!');
        }
        this.logger.debug(`Sending outbound message to service:`, { messageId: message.id, service });
        const keys = {
            recipientKeys: service.recipientKeys,
            routingKeys: service.routingKeys || [],
            senderKey,
        };
        // Set return routing for message if requested
        if (returnRoute) {
            message.setReturnRouting(TransportDecorator_1.ReturnRouteTypes.all);
        }
        const outboundPackage = await this.packMessage({ message, keys, endpoint: service.serviceEndpoint });
        await this.outboundTransporter.sendMessage(outboundPackage);
    }
};
MessageSender = __decorate([
    tsyringe_1.scoped(tsyringe_1.Lifecycle.ContainerScoped),
    __param(2, tsyringe_1.inject(constants_1.InjectionSymbols.MessageRepository)),
    __param(3, tsyringe_1.inject(constants_1.InjectionSymbols.Logger)),
    __metadata("design:paramtypes", [EnvelopeService_1.EnvelopeService,
        TransportService_1.TransportService, Object, Object])
], MessageSender);
exports.MessageSender = MessageSender;
function isDidCommTransportQueue(serviceEndpoint) {
    return serviceEndpoint === constants_1.DID_COMM_TRANSPORT_QUEUE;
}
exports.isDidCommTransportQueue = isDidCommTransportQueue;
//# sourceMappingURL=MessageSender.js.map